'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _require = require('eslint-plugin-react/lib/util/variable');

var findVariable = _require.findVariable;
var variablesInScope = _require.variablesInScope;

var htmlTags = require('html-tags');
var svgTags = require('svg-tags');

var standardTags = new Set(htmlTags.concat(svgTags));

/**
 * Check the factory function is imported if JSX is being used
 */

var factoryInScopeRule = function factoryInScopeRule(context) {
  var config = context.options[0] || {};
  var name = config.pragma || 'JSX';
  return {
    JSXOpeningElement: function JSXOpeningElement(node) {
      var variables = variablesInScope(context);
      if (findVariable(variables, name)) return;
      context.report({
        message: '\'' + name + '\' must be in scope when using JSX syntax',
        node: node
      });
    }
  };
};

factoryInScopeRule.schema = [{
  type: 'object',
  properties: {
    pragma: { type: 'string' }
  },
  additionalProperties: false
}];

/**
 * Prevent the JSX factory function from being marked as unused
 * if it will be in the compiled source
 */

var usesFactoryRule = function usesFactoryRule(context) {
  var config = context.options[0] || {};
  var id = config.pragma || 'JSX';
  return {
    JSXElement: function JSXElement() {
      context.markVariableAsUsed(id);
    }
  };
};

usesFactoryRule.schema = factoryInScopeRule.schema;

/**
 * Prevent variables used in JSX to be marked as unused
 */

var markUsedRule = function markUsedRule(context) {
  return {
    JSXOpeningElement: function JSXOpeningElement(node) {
      var name = node.name;
      if (name.type == 'JSXMemberExpression') name = name.object;
      if (name.type == 'JSXNamespacedName') name = name.namespace;
      context.markVariableAsUsed(name.name);
    },
    JSXAttribute: function JSXAttribute(attr) {
      if (attr.value == null) context.markVariableAsUsed(attr.name.name);
    }
  };
};

/**
 * Disallow undeclared variables in JSX
 */

var noUndefRule = function noUndefRule(context) {
  return {
    JSXOpeningElement: function JSXOpeningElement(node) {
      var name = node.name;
      if (name.type == 'JSXMemberExpression') name = name.object;
      if (name.type == 'JSXNamespacedName') name = name.namespace;
      var variables = variablesInScope(context);
      node.attributes.forEach(function (attr) {
        if (attr.value == null) checkDefined(context, variables, attr.name);
      });
      if (!standardTags.has(name.name)) checkDefined(context, variables, name);
    }
  };
};

var checkDefined = function checkDefined(context, variables, node) {
  if (findVariable(variables, node.name)) return;
  context.report({
    message: '\'' + node.name + '\' is not defined',
    node: node
  });
};

var rules = {
  'uses-factory': usesFactoryRule,
  'factory-in-scope': factoryInScopeRule,
  'mark-used-vars': markUsedRule,
  'no-undef': noUndefRule
};

// enable all by default
var rulesConfig = {
  'uses-factory': 1,
  'factory-in-scope': 1,
  'mark-used-vars': 1,
  'no-undef': 1
};

exports.rules = rules;
exports.rulesConfig = rulesConfig;

